﻿using System;
using System.Collections.Generic;
using System.Text;

using System.IO;
using System.Xml.Serialization;

namespace MyIllustPuzzle
{
    public partial class CC
    {
        public const int B_VALUE = 10;
        public const char SEPARATOR = '.';

        /// <summary> [CanGetList]メソッド([GetList])
        /// </summary>
        /// <param name="vfile"></param>
        /// <param name="vrow"></param>
        /// <returns>true:読出し成功</returns>
        public bool CanGetList(string vfile, ref List<string> vrow)
        {
            try
            {
                //*** ファイルの存在確認
                if (!File.Exists(vfile))
                {
                    ShowMsg("指定ファイルが見つかりません。", MSG_ERR);
                    return false;
                }

                using (var reader = new StreamReader(vfile, Encoding.GetEncoding("Shift_JIS")))
                {
                    vrow.Clear();
                    while (!reader.EndOfStream)
                    {
                        vrow.Add(reader.ReadLine());
                    }
                    reader.Close();
                }
                return true;
            }
            catch (Exception ex)
            {
                ShowMsg(ex.Message, MSG_ERR);
                return false;
            }
        }

        /// <summary> [CanPutList]メソッド([PutList])
        /// </summary>
        /// <param name="vfile"></param>
        /// <param name="vrow"></param>
        /// <returns>true:書込み成功</returns>
        public bool CanPutList(string vfile, List<string> vrow)
        {
            using (var writer = new StreamWriter(vfile, false, Encoding.GetEncoding("Shift_JIS")))
            {
                foreach (string row in vrow)
                {
                    writer.WriteLine(row);
                }
                writer.Close();
            }
            return true;
        }

        /// <summary> [CanReadXml]メソッド([ReadLogPar])
        /// </summary>
        /// <param name="vfile"></param>
        /// <param name="vpuz"></param>
        /// <returns>true:読出し成功</returns>
        public bool CanReadXml(string vfile, ref CPuzzle vpuz)
        {
            bool isOK;
            
            // ファイル読出し
            try
            {
                XmlSerializer serializer = new XmlSerializer(typeof(CPuzzle));
                FileStream stream1 = new FileStream(vfile, FileMode.Open);

                try
                {
                    vpuz = (CPuzzle)(serializer.Deserialize(stream1));
                    isOK = true;
                }
                catch (Exception ex1)
                {
                    ShowMsg(ex1.Message, MSG_ERR);
                    isOK = false;
                }
                finally
                {
                    stream1.Close();
                    stream1 = null;
                    serializer = null;
                }
            }
            catch (Exception ex2)
            {
                ShowMsg(ex2.Message, MSG_ERR);
                isOK = false;
            }
            return isOK;
        }

        /// <summary> [CanWriteXml]メソッド([WriteLogPar])
        /// </summary>
        /// <param name="vfile"></param>
        /// <param name="vpuz"></param>
        /// <returns>true:書込み成功</returns>
        public bool CanWriteXml(string vfile, CPuzzle vpuz)
        {
            bool isOK;
            try
            {
                XmlSerializer serializer = new XmlSerializer(typeof(CPuzzle));
                FileStream stream1 = new FileStream(vfile, FileMode.Create);
                try
                {
                    serializer.Serialize(stream1, vpuz);
                    isOK = true;
                }
                catch (Exception ex1)
                {
                    ShowMsg(ex1.Message, MSG_ERR);
                    isOK = false;
                }
                finally
                {
                    stream1.Close();
                    serializer = null;
                    stream1 = null;
                }
            }
            catch (Exception ex2)
            {
                ShowMsg(ex2.Message, MSG_ERR);
                isOK = false;
            }
            return isOK;
        }
        
    }

    [XmlRoot("LOGPAR-SETTINGS")]
    public class CPuzzle
    {
        private int cntCol = 0;
        private int cntRow = 0;

        [XmlArrayItem(Type = typeof(CColSet))]
        private List<CColSet> lstColSet = new List<CColSet>();
        [XmlArrayItem(Type = typeof(CRowSet))]
        private List<CRowSet> lstRowSet = new List<CRowSet>();

        [XmlElement("COUNT-COL")]
        public int CountCol
        {
            get { return cntCol; }
            set { cntCol = value; }
        }

        [XmlElement("COUNT-ROW")]
        public int CountRow
        {
            get { return cntRow; }
            set { cntRow = value; }
        }

        [XmlElement("COL-SETTINGS")]
        public List<CColSet> ColSet
        {
            get { return lstColSet; }
            set { lstColSet = value; }
        }

        [XmlElement("ROW-SETTINGS")]
        public List<CRowSet> RowSet
        {
            get { return lstRowSet; }
            set { lstRowSet = value; }
        }
    }

    [XmlRoot("COL-SETTINGS")]
    public class CColSet
    {
        private int colNum = 0;
        private string colData = String.Empty;

        [XmlElement("COL-NUMBER")]
        public int NumCol
        {
            get { return colNum; }
            set { colNum = value; }
        }

        [XmlElement("COL-DATA")]
        public string DataCol
        {
            get { return colData; }
            set { colData = value; }
        }
    }

    [XmlRoot("ROW-SETTINGS")]
    public class CRowSet
    {
        private int rowNum = 0;
        private string rowData = String.Empty;

        [XmlElement("ROW-NUMBER")]
        public int NumRow
        {
            get { return rowNum; }
            set { rowNum = value; }
        }

        [XmlElement("ROW-DATA")]
        public string DataRow
        {
            get { return rowData; }
            set { rowData = value; }
        }
    }

}
