﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;

namespace MyIllustPuzzle
{
    public partial class FormMain : Form
    {
        /// <summary> コンストラクター
        /// </summary>
        public FormMain()
        {
            InitializeComponent();
        }

        /// <summary> [CellState]列挙体
        ///  Cellの状態を表す列挙体
        /// </summary>
        private enum CellState
        {
            Unfixed,
            Yellow,
            Black
        }

        /// <summary> [CoRo]列挙体
        ///  行・列識別のための列挙体
        /// </summary>
        private enum CoRo
        {
            Column,
            Row
        }

        /// <summary> [SctCR]構造体
        ///  行・列の状態を表す構造体、_sctcol[],_sctrow[]として使う
        /// </summary>
        private struct SctCR
        {
            // 設定時に決まる因子
            public int cblock;                // 各行・列のブロック数
            public int[] cvalue;              // i番目ブロックの黒数
            public int ceffective;            // 有効数合計(各行・列の黒数合計+ブロック数-1)
            // ゲームの進行に伴って変化する因子
            public bool vdecided;             // 各行・列の確定(true), 未確定(false)
        }

        const int C_SISE = 9;                 // セルの塗色部サイズ

        //*** フィールド変数
        private readonly string CEL_BLK = ((int)(CellState.Black)).ToString();
        private readonly string CEL_YEL = ((int)(CellState.Yellow)).ToString();
        private Bitmap _bpic;
        private bool _cancel;                 // true：FormCancelのキャンセルボタンが押されたとき
        private bool _openFile = false;       // [btnOpen_Click]でtrueに設定される
        private bool _trouble;                // [IsChangedByTry4Detail]でtrueの判定(Ver1.0.1で追加)
        private CellState[,] _cells;          // [InitializeCellState]で初期化
        private DateTime _start;              //
        private FormCancel _cclfrm;           // FormCancelのインスタンス､[btnExecute_Click]でnew
        private int B;                        // セルのサイズ(CFileIOのB_VALUE) [Form_Load]で設定
        private int _count;                   // Try4での繰り返し回数
        private int _col;                     // 問題の列数、[Col]プロパティから取得
        private int _crnum;                   // Try4で実行中の行(列)番号 (Statusbar表示用)
        private int _decided = 0;             // 決定済みセル数
        private int _row;                     // 問題の行数、[Row]プロパティから取得
        private int _total;                   // 全セル数(_row×_col) [Form_Load]で取得
        private decimal _ts;                  // timespan(秒数)、[IsChangedByTry4]で使用
        private List<string> _lstCol;         // 列設定数値配列のリスト、ListColumnプロパティから取得
        private List<string> _lstRow;         // 行設定数値配列のリスト、ListRowプロパティから取得
        private SctCR[] _sctcol;              // InitializeCellStateメソッドで初期化
        private SctCR[] _sctrow;              // InitializeCellStateメソッドで初期化
        private string _columnrow;            // Try4で実行中の行(R)または列(C) (Statusbar表示用)
        private string _dir;                  // ファイルを保存するフォルダ
        private string _filename;             // [FileName]プロパティから取得
        // CellState型配列リスト _lstState：各行・列で取り得るCellState型配列のリスト
        private List<CellState[]> _lstState;  // [IsChangedByTry4Detail]で初期化, [ClearForm]でクリア

        #region "プロパティ"

        /// <summary> [Col]プロパティ：列数
        /// </summary>
        public int Col
        {
            set { _col = value; }
        }

        /// <summary> [Row]プロパティ：行数
        /// </summary>
        public int Row
        {
            set { _row = value; }
        }

        /// <summary> [FileName]プロパティ
        /// </summary>
        public string FileName
        {
            set { _filename = value; }
        }

        /// <summary> [Folder]プロパティ
        /// </summary>
        public string Folder
        {
            set { _dir = value; }
        }

        /// <summary> [ListColumn]プロパティ
        /// </summary>
        public List<String> ListColumn
        {
            set { _lstCol = value; }
        }

        /// <summary> [ListRow]プロパティ
        /// </summary>
        public List<String> ListRow
        {
            set { _lstRow = value; }
        }

        # endregion

        /// <summary> [確認]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCheck_Click(object sender, EventArgs e)
        {
            CheckResult();
        }

        /// <summary> [クリア]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClear_Click(object sender, EventArgs e)
        {
            ClearForm();
        }

        /// <summary> [実行]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnExecute_Click(object sender, EventArgs e)
        {
            // cellsの初期化
            if (_openFile)             // [開く]の後に実行される場合
                _openFile = false;
            else
                InitializeCellState();

            DateTime begin = DateTime.Now;

            btnExecute.Enabled = false;

            // [キャンセル]ダイアログを表示する
            _cclfrm = new FormCancel(this.Left, this.Top);
            _cclfrm.Owner = this;            // ダイアログのオーナーを設定
            _cclfrm.Show();

            _cancel = false;
            _trouble = false;

            try
            {
                Try1();
                Try2();

                _count = 0;
                _ts = 0;              // timespan
                string msg = String.Empty;
                bool changed;         // IsChangedByTry3,4を受けて、ループ継続判定のためのフラグ
                while (true)
                {
                    if (5 <= _ts)
                        _ts += 5;
                    else if (1 <= _ts)
                        _ts += 1;
                    else if (_ts == 0.5M)
                        _ts += 0.5M;
                    else
                        _ts += 0.1M;
                    //_ts += 1;         // timespanを1秒大きくする
                    changed = true;
                    while (changed)
                    {
                        // セル決定が継続されている間は、処理を続ける
                        _count += 1;
                        changed = IsChangedByTry3();
                        changed = IsChangedByTry4() || changed;

                        if (_total == _decided || _cancel || _trouble) break;
                    }
                    if (_total == _decided || _cancel || _trouble) break;
                }

                // [キャンセル]ダイアログを閉じる
                _cclfrm.Close();

                if (_trouble)     // Ver1.0.1で追加
                {
                    btnSave.Enabled = false;
                    btnClear.Enabled = false;
                }
                else
                {
                    msg = String.Empty;
                    if (_cancel)
                        msg = "処理を中断しました。" + CC.NL + CC.NL;
                    TimeSpan ts = DateTime.Now - begin;
                    int ihour = (int)(ts.TotalHours);
                    int imin = ts.Minutes;
                    int isec = ts.Seconds;
                    string sts = String.Empty;
                    if (ihour > 0)
                    {
                        sts = String.Format("{0}時間{1}分{2}秒", ihour, imin, isec);
                    }
                    else if (imin > 0)
                    {

                        sts = String.Format("{0}分{1}秒", imin, isec);
                    }
                    else
                    {
                        decimal dsec = isec + Convert.ToDecimal(ts.Milliseconds / 10) / 100;
                        sts = dsec.ToString() + "秒";
                    }
                    msg += "ループ" + _count.ToString() + "回    " + CC.NL + sts;
                    CC.ShowMsg(msg, CC.MSG_INF);
                    if (!_cancel)
                    {
                        btnCheck.Enabled = true;
                        btnCheck.Focus();
                    }
                }
            }
            catch (Exception ex)
            {
                // [キャンセル]ダイアログを閉じる
                _cclfrm.Close();

                string msg2 = "エラーが発生しました。" + CC.NL + CC.NL + ex.Message;
                CC.ShowMsg(msg2, CC.MSG_ERR);
            }
        }

        /// <summary> [開く]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOpen_Click(object sender, EventArgs e)
        {
            string file = _dir + _filename + ".csv";
            // ファイルの存在確認
            if (!File.Exists(file))
            {
                CC.ShowMsg("この問題は未だ保存されていません。", CC.MSG_EXC);
                return;
            }

            // 状態の初期化
            ClearForm();

            // ファイル読み出し
            List<string> rowlist = new List<string>();
            string srow = String.Empty;
            var occ = new CC();
            if (!occ.CanGetList(file, ref rowlist)) return;
            
            _decided = 0;
            bool allFixed = true;     // 全セルが決定済みであるかを示すフラグ
            int countlist = rowlist.Count;
            string[] lastitem = rowlist[countlist - 1].Split(',');
            // 最終行が行・列数の場合は、その行は読まない(countlist を１減らす)
            if (lastitem.Length == 2) countlist = countlist - 1;
            for (int i = 0; i < countlist; i++)
            {
                string[] column = rowlist[i].Split(',');
                for (int j = 0; j < column.Length; j++)
                {
                    if (column[j] == CEL_BLK)
                    {
                        SetCellColor(i, j, Color.Black);
                        _decided++;            // 決定済み要素数を1つ増やす
                    }
                    else if (column[j] == CEL_YEL)
                    {
                        SetCellColor(i, j, Color.Yellow);
                        _decided++;           // 決定済み要素数を1つ増やす
                    }
                    else
                    {
                        SetCellColor(i, j, SystemColors.Control);
                        allFixed = false;   // 未決定のセルが存在する
                    }

                    _cells[i, j] = (CellState)(Convert.ToInt32(column[j]));
                }
            }
            if (allFixed)
            {
                // 全セルが決定済みの場合
                btnExecute.Enabled = false;
                btnCheck.Enabled = true;
            }
            else
            {
                // 未決定のセルが存在する場合
                btnExecute.Enabled = true;
                btnCheck.Enabled = false;
                _openFile = true;
            }
        }

        /// <summary> [保存]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, EventArgs e)
        {
            if (_decided == 0)
            {
                CC.ShowMsg("保存するデータがありません。", CC.MSG_EXC);
                return;
            }
            
            string ofile = _dir + _filename + ".csv";
            string msg = "この結果は保存済みです。" + CC.NL +
                "上書き保存しますが、よろしいですか？";
            // ファイルの存在確認
            if (File.Exists(ofile))
            {
                if (CC.MsgCanceled(msg)) return;
            }

            List<string> rowlist = new List<string>();

            for (int i = 0; i < _row; i++)
            {
                string srow = ((int)_cells[i, 0]).ToString();
                for (int j = 1; j < _col; j++)
                {
                    srow += "," + ((int)_cells[i, j]).ToString();
                }
                rowlist.Add(srow);
            }
            rowlist.Add(_row.ToString() + "," + _col.ToString());
            var occ = new CC();
            if (occ.CanPutList(ofile, rowlist))
            {
                CC.ShowMsg("データを保存しました。", CC.MSG_INF);
            }
            occ = null;
            btnOpen.Enabled = true;
        }

        /// <summary> [Form_Load]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Load(object sender, EventArgs e)
        {
            B = CC.B_VALUE;   // 要素サイズの設定
            this.Text = "[" + _filename + "]"
                + " (" + _row.ToString() + "行" + _col.ToString() + "列" + ")";

            SetForm();
            _cells = new CellState[_row, _col];
            SetSctCR();            // SctCR構造体の初期化
            _total = _col * _row;

            // ファイルの存在確認
            string ofile = _dir + _filename + ".csv";
            if (!File.Exists(ofile)) btnOpen.Enabled = false;

            btnCheck.Enabled = false;
            stlbl1.Width = sstrip1.Width - 30;
        }


        /// <summary> [IsChangedByTry3]メソッド：四辺をチェックする
        ///  [btnExecute_Click]から呼出し
        /// </summary>
        /// <returns>変更があった場合にtrueを返す</returns>
        private bool IsChangedByTry3()
        {
            bool changed = false;
            bool crfinished;
            int block;

            // gridの左右端のチェック
            for (int i1 = 0; i1 < _row; i1++)
            {
                if (_sctrow[i1].vdecided) continue;
                int colx;

                // gridの左端のチェック
                if (_cells[i1, 0] == CellState.Black)
                {
                    colx = 0;
                    block = 0;
                    for (int i2 = 0; i2 < _sctrow[i1].cblock; i2++)
                    {
                        for (int i3 = 0; i3 < _sctrow[i1].cvalue[block]; i3++)
                        {
                            if (colx > _col - 1) break;   // Ver0.4.0で追加
                            changed = IsColorChanged(i1, colx, CellState.Black) || changed;
                            colx += 1;
                        }
                        if (colx > _col - 1) break;
                        changed = IsColorChanged(i1, colx, CellState.Yellow) || changed;
                        block += 1;
                        colx += 1;
                        if (colx > _col - 1) break;
                        if (block > _sctrow[i1].cblock - 1)
                        {
                            for (int i4 = colx; i4 >= 0; i4--)
                            { changed = IsColorChanged(i1, i4, CellState.Yellow) || changed; }
                            crfinished = IsCrFinished(i1, CoRo.Row);
                            break;
                        }
                        if (_cells[i1, colx] == CellState.Unfixed)
                        { break; }
                        else if (_cells[i1, colx] == CellState.Yellow)
                        {
                            while (_cells[i1, colx] == CellState.Yellow)
                            {
                                if (colx == _col - 1) break;
                                colx += 1;
                            }
                            if (_cells[i1, colx] == CellState.Unfixed) break;
                        }
                    }
                }
                else if (_cells[i1, 0] == CellState.Yellow)
                {
                    colx = 1;
                    while (_cells[i1, colx] == CellState.Yellow)
                    { colx += 1; }
                    if (_cells[i1, colx] == CellState.Black)
                    {
                        block = 0;
                        for (int i2 = 0; i2 < _sctrow[i1].cblock; i2++)
                        {
                            for (int i3 = 0; i3 < _sctrow[i1].cvalue[block]; i3++)
                            {
                                if (colx > _col - 1) break;
                                changed = IsColorChanged(i1, colx, CellState.Black) || changed;
                                colx += 1;
                            }
                            if (colx > _col - 1) break;
                            changed = IsColorChanged(i1, colx, CellState.Yellow) || changed;
                            block += 1;
                            colx += 1;
                            if (colx > _col - 1) break;
                            if (block > _sctrow[i1].cblock - 1)
                            {
                                for (int i4 = colx; i4 >= 0; i4--)
                                { changed = IsColorChanged(i1, i4, CellState.Yellow) || changed; }
                                crfinished = IsCrFinished(i1, CoRo.Row);
                                break;
                            }
                            if (_cells[i1, colx] == CellState.Unfixed)
                            { break; }
                            else if (_cells[i1, colx] == CellState.Yellow)
                            {
                                while (_cells[i1, colx] == CellState.Yellow)
                                {
                                    if (colx == _col - 1) break;
                                    colx += 1;
                                }
                                if (_cells[i1, colx] == CellState.Unfixed) break;
                            }
                        }
                    }
                }

                // gridの右端のチェック
                if (_cells[i1, _col - 1] == CellState.Black)
                {
                    colx = _col - 1;

                    block = _sctrow[i1].cblock - 1;
                    for (int i2 = _sctrow[i1].cblock - 1; i2 >= 0; i2--)
                    {
                        for (int i3 = 0; i3 < _sctrow[i1].cvalue[block]; i3++)
                        {
                            if (colx < 0) break;
                            changed = IsColorChanged(i1, colx, CellState.Black) || changed;
                            colx -= 1;
                        }
                        if (colx < 0) break;
                        changed = IsColorChanged(i1, colx, CellState.Yellow) || changed;
                        block -= 1;
                        colx -= 1;
                        if (colx < 0) break;
                        if (block < 0)
                        {
                            for (int i4 = colx; i4 >= 0; i4--)
                            { changed = IsColorChanged(i1, i4, CellState.Yellow) || changed; }
                            crfinished = IsCrFinished(i1, CoRo.Row);
                            break;
                        }
                        if (_cells[i1, colx] == CellState.Unfixed)
                        { break; }
                        else if (_cells[i1, colx] == CellState.Yellow)
                        {
                            while (_cells[i1, colx] == CellState.Yellow)
                            {
                                if (colx == 0) break;
                                colx -= 1;
                            }
                            if (_cells[i1, colx] == CellState.Unfixed) break;
                        }
                    }
                }
                else if (_cells[i1, _col - 1] == CellState.Yellow)
                {
                    colx = _col - 2;
                    while (_cells[i1, colx] == CellState.Yellow)
                    { colx -= 1; }
                    if (_cells[i1, colx] == CellState.Black)
                    {
                        block = _sctrow[i1].cblock - 1;
                        for (int i2 = _sctrow[i1].cblock - 1; i2 >= 0; i2--)
                        {
                            for (int i3 = 0; i3 < _sctrow[i1].cvalue[block]; i3++)
                            {
                                if (colx < 0) break;
                                changed = IsColorChanged(i1, colx, CellState.Black) || changed;
                                colx -= 1;
                            }
                            if (colx < 0) break;
                            changed = IsColorChanged(i1, colx, CellState.Yellow) || changed;
                            block -= 1;
                            colx -= 1;
                            if (colx < 0) break;
                            if (block < 0)
                            {
                                for (int i4 = colx; i4 >= 0; i4--)
                                { changed = IsColorChanged(i1, i4, CellState.Yellow) || changed; }
                                crfinished = IsCrFinished(i1, CoRo.Row);
                                break;
                            }
                            if (_cells[i1, colx] == CellState.Unfixed)
                            { break; }
                            else if (_cells[i1, colx] == CellState.Yellow)
                            {
                                while (_cells[i1, colx] == CellState.Yellow)
                                {
                                    if (colx == 0) break;
                                    colx -= 1;
                                }
                                if (_cells[i1, colx] == CellState.Unfixed)
                                { break; }
                            }
                        }
                    }
                }
            }

            // gridの上下端のチェック
            for (int j1 = 0; j1 < _col; j1++)
            {
                if (_sctcol[j1].vdecided) continue;
                int rowx;

                // gridの上端のチェック
                if (_cells[0, j1] == CellState.Black)
                {
                    rowx = 0;
                    block = 0;
                    for (int j2 = 0; j2 < _sctcol[j1].cblock; j2++)
                    {
                        for (int j3 = 0; j3 < _sctcol[j1].cvalue[block]; j3++)
                        {
                            if (rowx > _row - 1) break;
                            changed = IsColorChanged(rowx, j1, CellState.Black) || changed;
                            rowx += 1;
                        }
                        if (rowx > _row - 1) break;
                        changed = IsColorChanged(rowx, j1, CellState.Yellow) || changed;
                        block += 1;
                        rowx += 1;
                        if (rowx > _row - 1) break;
                        if (block > _sctcol[j1].cblock - 1)
                        {
                            for (int j4 = rowx; j4 >= 0; j4--)
                            { changed = IsColorChanged(j4, j1, CellState.Yellow) || changed; }
                            crfinished = IsCrFinished(j1, CoRo.Column);
                            break;
                        }
                        if (_cells[rowx, j1] == CellState.Unfixed)
                        { break; }
                        else if (_cells[rowx, j1] == CellState.Yellow)
                        {
                            while (_cells[rowx, j1] == CellState.Yellow)
                            {
                                if (rowx == _row - 1) break;
                                rowx += 1;
                            }
                            if (_cells[rowx, j1] == CellState.Unfixed) break;
                        }
                    }
                }
                else if (_cells[0, j1] == CellState.Yellow)
                {
                    rowx = 1;
                    while (_cells[rowx, j1] == CellState.Yellow)
                    { rowx += 1; }
                    if (_cells[rowx, j1] == CellState.Black)
                    {
                        block = 0;
                        for (int j2 = 0; j2 < _sctcol[j1].cblock; j2++)
                        {
                            for (int j3 = 0; j3 < _sctcol[j1].cvalue[block]; j3++)
                            {
                                if (rowx > _row - 1) break;
                                changed = IsColorChanged(rowx, j1, CellState.Black) || changed;
                                rowx += 1;
                            }
                            if (rowx > _row - 1) break;
                            changed = IsColorChanged(rowx, j1, CellState.Yellow) || changed;
                            block += 1;
                            rowx += 1;
                            if (rowx > _row - 1) break;
                            if (block > _sctcol[j1].cblock - 1)
                            {
                                for (int j4 = rowx; j4 >= 0; j4--)
                                { changed = IsColorChanged(j4, j1, CellState.Yellow) || changed; }
                                crfinished = IsCrFinished(j1, CoRo.Column);
                                break;
                            }
                            if (_cells[rowx, j1] == CellState.Unfixed)
                            { break; }
                            else if (_cells[rowx, j1] == CellState.Yellow)
                            {
                                while (_cells[rowx, j1] == CellState.Yellow)
                                {
                                    if (rowx == _row - 1) break;
                                    rowx += 1;
                                }
                                if (_cells[rowx, j1] == CellState.Unfixed) break;
                            }
                        }
                    }
                }

                // gridの下端のチェック
                if (_cells[_row - 1, j1] == CellState.Black)
                {
                    rowx = _row - 1;
                    block = _sctcol[j1].cblock - 1;
                    for (int j2 = _sctcol[j1].cblock - 1; j2 >= 0; j2--)
                    {
                        for (int j3 = 0; j3 < _sctcol[j1].cvalue[block]; j3++)
                        {
                            if (rowx < 0) break;
                            changed = IsColorChanged(rowx, j1, CellState.Black) || changed;
                            rowx -= 1;
                        }
                        if (rowx < 0) break;
                        changed = IsColorChanged(rowx, j1, CellState.Yellow) || changed;
                        block -= 1;
                        rowx -= 1;
                        if (rowx < 0) break;
                        if (block < 0)
                        {
                            for (int j4 = rowx; j4 >= 0; j4--)
                            { changed = IsColorChanged(j4, j1, CellState.Yellow) || changed; }
                            crfinished = IsCrFinished(j1, CoRo.Column);
                            break;
                        }
                        if (_cells[rowx, j1] == CellState.Unfixed)
                        { break; }
                        else if (_cells[rowx, j1] == CellState.Yellow)
                        {
                            while (_cells[rowx, j1] == CellState.Yellow)
                            {
                                if (rowx == 0) break;
                                rowx -= 1;
                            }
                            if (_cells[rowx, j1] == CellState.Unfixed) break;
                        }
                    }
                }
                else if (_cells[_row - 1, j1] == CellState.Yellow)
                {
                    rowx = _row - 2;
                    while (_cells[rowx, j1] == CellState.Yellow)
                    { rowx -= 1; }
                    if (_cells[rowx, j1] == CellState.Black)
                    {
                        block = _sctcol[j1].cblock - 1;
                        for (int j2 = _sctcol[j1].cblock - 1; j2 >= 0; j2--)
                        {
                            for (int j3 = 0; j3 < _sctcol[j1].cvalue[block]; j3++)
                            {
                                if (rowx < 0) break;
                                changed = IsColorChanged(rowx, j1, CellState.Black) || changed;
                                rowx -= 1;
                            }
                            if (rowx < 0) break;
                            changed = IsColorChanged(rowx, j1, CellState.Yellow) || changed;
                            block -= 1;
                            rowx -= 1;
                            if (rowx < 0) break;
                            if (block < 0)
                            {
                                for (int j4 = rowx; j4 >= 0; j4--)
                                { changed = IsColorChanged(j4, j1, CellState.Yellow) || changed; }
                                crfinished = IsCrFinished(j1, CoRo.Column);
                                break;
                            }
                            if (_cells[rowx, j1] == CellState.Unfixed)
                            { break; }
                            else if (_cells[rowx, j1] == CellState.Yellow)
                            {
                                while (_cells[rowx, j1] == CellState.Yellow)
                                {
                                    if (rowx == 0) break;
                                    rowx -= 1;
                                }
                                if (_cells[rowx, j1] == CellState.Unfixed) break;
                            }
                        }
                    }
                }
            }
            return changed;
        }

        /// <summary> [IsChangedByTry4]メソッド：
        ///  [btnExecute_Click]から呼出し
        /// </summary>
        /// <returns>変更があった場合にtrueを返す</returns>
        private bool IsChangedByTry4()
        {
            bool changed = false;   // 変更有無のフラグ

            // 各行の処理
            _columnrow = "R";       // Statusbar用行・列識別表示値
            for (int i1 = 0; i1 < _row; i1++)
            {
                _crnum = i1 + 1;    // Statusbar表示用行Noの取得
                if (!IsCrFinished(i1, CoRo.Row))  // 該当行に未決の要素がある場合
                {
                    changed = IsChangedByTry4Detail(i1, CoRo.Row) || changed;
                    if (_cancel) return false;
                }
                UpdateStatusbar();
            }

            // 各列の処理
            _columnrow = "C";       // Statusbar用行・列識別表示値
            for (int j1 = 0; j1 < _col; j1++)
            {
                _crnum = j1 + 1;    // Statusbar表示用列Noの取得
                if (!IsCrFinished(j1, CoRo.Column))  // 該当列に未決の要素がある場合
                {
                    changed = IsChangedByTry4Detail(j1, CoRo.Column) || changed;
                    if (_cancel) return false;
                }
                UpdateStatusbar();
            }
            return changed;
        }

        /// <summary> [IsChangedByTry4Detail]メソッド
        ///  [IsChangedByTry4]から呼出し
        /// </summary>
        /// <param name="vn">行・列の値</param>
        /// <param name="vcr">enum CoRo</param>
        /// <returns>変更があった場合にtrueを返す</returns>
        private bool IsChangedByTry4Detail(int vn, CoRo vcr)
        {
            bool changed = false;

            
            _lstState = new List<CellState[]>(); 
            var scr = new SctCR();
            CellState[] basestate;    // このメソッドが呼出されたときの対象行・列のセル状態
            if (vcr == CoRo.Column)
            {
                // 列のCheck
                scr = _sctcol[vn];
                basestate = new CellState[_row];
                for (int k1 = 0; k1 < _row; k1++)
                {
                    basestate[k1] = _cells[k1, vn];
                }
            }
            else
            {
                // 行のCheck
                scr = _sctrow[vn];
                basestate = new CellState[_col];
                for (int k2 = 0; k2 < _col; k2++)
                {
                    basestate[k2] = _cells[vn, k2];
                }
            }

            if (scr.ceffective < 0) return false;    // ceffective：各行・列の黒要素数合計+blockの数-1

            int countcr = (vcr == CoRo.Column) ? _row : _col;   // 行・列の数
            CellState[] state1 = new CellState[countcr];
            for (int i1 = 0; i1 < countcr; i1++)
            {
                state1[i1] = CellState.Yellow;       // 全てYellowにして、Try4GetCombiに渡す
            }

            _start = DateTime.Now;                   // timespanのカウント開始
            
            // 全ての組合せを _lstState に取得する
            int margin = countcr - scr.ceffective;              // 該当行・列の動かし代
            bool result = true;
            Try4GetCombi(scr, countcr, 0, margin, 0, state1, basestate, ref result);

            if (!result) return false;               // timespanを超えたとき
            if (_cancel) return false;               // [キャンセル]ボタンがクリックされたとき            

            int cntlist1 = _lstState.Count;
            if (cntlist1 > 0)
            {
                // 不適正なListを削除する(この部分の処理に時間がかかる)
                for (int i2 = cntlist1 - 1; i2 >= 0; i2 -= 1)
                {
                    if (vcr == CoRo.Column)
                    {
                        for (int i31 = 0; i31 < countcr; i31++)
                        {
                            if (_cells[i31, vn] != CellState.Unfixed)
                            {
                                if (_lstState[i2][i31] != _cells[i31, vn])
                                {
                                    _lstState.RemoveAt(i2);
                                    break;
                                }
                            }
                        }
                    }
                    else
                    {
                        for (int i32 = 0; i32 < countcr; i32++)
                        {
                            if (_cells[vn, i32] != CellState.Unfixed)
                            {
                                if (_lstState[i2][i32] != _cells[vn, i32])
                                {
                                    _lstState.RemoveAt(i2);
                                    break;
                                }
                            }
                        }
                    }
                }

                // 異常事態(_lstState.Count=0)が発生したときの対処 ←Ver1.0.1で追加
                if (_lstState.Count == 0)
                {
                    string msg = "異常が発生しました。" + CC.NL
                        + "入力されたデータに問題がないか、確認をしてください。";
                    CC.ShowMsg(msg, CC.MSG_EXC);
                    _trouble = true;
                    return false;
                }

                // 全組合せで同一セル状態となるセルを見つける
                for (int i4 = 0; i4 < countcr; i4++)
                {
                    // 決定済みセルは処理しない
                    if (vcr == CoRo.Column)
                    {
                        if (_cells[i4, vn] != CellState.Unfixed) continue;
                    }
                    else
                    {
                        if (_cells[vn, i4] != CellState.Unfixed) continue;
                    }

                    bool isOK = true;
                    CellState stateA = _lstState[0][i4];
                    // 全ての組合せでセル状態が同一かを見分ける⇒同一であれば、(isOK = true) 
                    for (int i5 = 1; i5 < _lstState.Count; i5++)
                    {
                        if (stateA != _lstState[i5][i4])  // 異なったセル状態の場合
                        {
                            isOK = false;
                            break;
                        }
                    }
                    if (isOK)
                    {
                        if (vcr == CoRo.Column)
                        {
                            if (stateA == CellState.Yellow)
                                 changed = IsColorChanged(i4, vn, CellState.Yellow) || changed;
                            else
                                changed = IsColorChanged(i4, vn, CellState.Black) || changed;
                        }
                        else
                        {
                            if (stateA == CellState.Yellow)
                                changed = IsColorChanged(vn, i4, CellState.Yellow) || changed;
                            else
                                changed = IsColorChanged(vn, i4, CellState.Black) || changed;
                        }
                    }
                }
            }
            return changed;
        }

        /// <summary> [IsColorChanged]：各セルを、CellStateの状態に応じて塗色する
        /// </summary>
        /// <param name="vx">列No</param>
        /// <param name="vy">行No</param>
        /// <param name="vstat">CellState</param>
        /// <returns>塗色された(変更された)場合、trueを返す</returns>
        private bool IsColorChanged(int vx, int vy, CellState vstate)
        {
            bool changed = false;
            if (vx < 0 || vy < 0) return false;

            if (_cells[vx, vy] == CellState.Unfixed)  // 対象は未塗色セルのみ
            {
                _cells[vx, vy] = vstate;
                _decided += 1;
                if (_cells[vx, vy] == CellState.Black)
                {
                    SetCellColor(vx, vy, Color.Black);
                }
                else if (_cells[vx, vy] == CellState.Yellow)
                {
                    SetCellColor(vx, vy, Color.Yellow);
                }
                UpdateStatusbar();
                changed = true;
            }
            return changed;
        }

        /// <summary> [IsCrFinished]メソッド：対象行・列のセルが全て既決であるかを返す
        ///  [IsChangedByTry4]から呼出し
        /// </summary>
        /// <param name="vn">行・列の番号</param>
        /// <param name="vcr">enum CoRo</param>
        /// <returns>該当行・列のセルが全て既決の場合、trueを返す</returns>
        private bool IsCrFinished(int vn, CoRo vcr)
        {
            bool changed;    // IsColorChangedメソッドの戻り値を受けるbool変数
            if (vcr == CoRo.Column)    // 列の場合
            {
                if (_sctcol[vn].vdecided) return true;
                int nblack1 = _sctcol[vn].ceffective - _sctcol[vn].cblock + 1;  // 黒数合計
                int count1 = 0;
                // 列内の黒塗色セルの数を取得
                for (int j1 = 0; j1 < _row; j1++)
                {
                    if (_cells[j1, vn] == CellState.Black) count1++;
                }
                if (nblack1 == count1)    // 列内黒塗色セル数が設定値に達した場合
                {
                    for (int j2 = 0; j2 < _row; j2++)
                    {
                        // 列内未決セルを黄色に塗色する
                        if (_cells[j2, vn] == CellState.Unfixed)
                            changed = IsColorChanged(j2, vn, CellState.Yellow);
                    }
                    _sctcol[vn].vdecided = true;
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else              // 行の場合
            {
                if (_sctrow[vn].vdecided) return true;
                int nblack2 = _sctrow[vn].ceffective - _sctrow[vn].cblock + 1;  // 黒数合計
                int count2 = 0;
                // 行内の黒塗色セルの数を取得
                for (int i1 = 0; i1 < _col; i1++)
                {
                    if (_cells[vn, i1] == CellState.Black) count2++;
                }
                if (nblack2 == count2)    // 行内黒塗色セル数が設定値に達した場合
                {
                    for (int i2 = 0; i2 < _col; i2++)
                    {
                        // 行内未決セルを黄色に塗色する
                        if (_cells[vn, i2] == CellState.Unfixed)
                            changed = IsColorChanged(vn, i2, CellState.Yellow);
                    }
                    _sctrow[vn].vdecided = true;
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }

        /// <summary> [CheckResult]メソッド
        ///  [btnCheck_Click]から呼出し
        /// </summary>
        private void CheckResult()
        {
            bool isOK = false;

            int val;
            // 行のチェック
            List<string> ngrow = new List<string>();
            string cr = "R";
            for (int i1 = 0; i1 < _row; i1++)
            {
                int col = 0;
                int block1 = _sctrow[i1].cblock;
                if (block1 == 0)   // 通常、この状態はない
                {
                    for (int i3 = 0; i3 < _col; i3++)
                    {
                        if (_cells[i1, i3] != CellState.Yellow)
                        {
                            ngrow.Add(cr + (i1 + 1).ToString());
                            break;
                        }
                    }
                }
                else
                {
                    bool isBreak1 = false;
                    for (int i2 = 0; i2 < block1; i2++)
                    {
                        while (col < _col && _cells[i1, col] == CellState.Yellow)
                        {
                            if (_cells[i1, col] == CellState.Unfixed)
                            {
                                isBreak1 = true;
                                break;
                            }
                            col += 1;
                        }
                        if (isBreak1)
                        {
                            ngrow.Add(cr + (i1 + 1).ToString());
                            break;
                        }
                        val = 0;
                        while (col < _col && _cells[i1, col] == CellState.Black)
                        {
                            if (_cells[i1, col] == CellState.Unfixed)
                            {
                                isBreak1 = true;
                                break;
                            }
                            col += 1;
                            val += 1;
                        }

                        if (isBreak1 || val != _sctrow[i1].cvalue[i2])
                        {
                            isBreak1 = true;
                            ngrow.Add(cr + (i1 + 1).ToString());
                            break;
                        }
                    }

                    // ブロックCheckが終わった後のセルのチェック
                    if (!isBreak1)
                    {
                        while (col < _col)
                        {
                            if (_cells[i1, col] == CellState.Black)
                            {
                                ngrow.Add(cr + (i1 + 1).ToString());
                                break;
                            }
                            col += 1;
                        }
                    }
                }
            }

            // 不良行の表示
            int cntng = ngrow.Count;
            if (cntng > 0)
            {
                string result1 = "不良行があります。" + Environment.NewLine + " " + ngrow[0];
                if (cntng > 1)
                {
                    for (int k1 = 1; k1 < cntng; k1++)
                    {
                        result1 += Environment.NewLine + " " + ngrow[k1];
                    }
                }
                CC.ShowMsg(result1, CC.MSG_EXC);
            }
            else
            {
                isOK = true;
            }

            // 列のチェック
            List<string> ngcol = new List<string>();
            cr = "C";
            for (int j1 = 0; j1 < _col; j1++)
            {
                int row = 0;
                int block2 = _sctcol[j1].cblock;
                if (block2 == 0)   // 通常、この状態はない
                {
                    for (int j3 = 0; j3 < _row; j3++)
                    {
                        if (_cells[j1, j3] != CellState.Yellow)
                        {
                            ngcol.Add(cr + (j1 + 1).ToString());
                            break;
                        }
                    }
                }
                else
                {
                    bool isBreak2 = false;
                    for (int j2 = 0; j2 < block2; j2++)
                    {
                        while (row < _row && _cells[row, j1] == CellState.Yellow)
                        {
                            if (_cells[row, j1] == CellState.Unfixed)
                            {
                                isBreak2 = true;
                                break;
                            }
                            row += 1;
                        }
                        if (isBreak2)
                        {
                            ngcol.Add(cr + (j1 + 1).ToString());
                            break;
                        }
                        val = 0;
                        while (row < _row && _cells[row, j1] == CellState.Black)
                        {
                            if (_cells[row, j1] == CellState.Unfixed)
                            {
                                isBreak2 = true;
                                break;
                            }
                            row += 1;
                            val += 1;
                        }
                        if (isBreak2 || val != _sctcol[j1].cvalue[j2])
                        {
                            isBreak2 = true;
                            ngcol.Add(cr + (j1 + 1).ToString());
                            break;
                        }
                    }

                    // ブロックCheckが終わった後のセルのチェック
                    if (!isBreak2)
                    {
                        while (row < _row)
                        {
                            if (_cells[row, j1] == CellState.Black)
                            {
                                ngcol.Add(cr + (j1 + 1).ToString());
                                break;
                            }
                            row += 1;
                        }
                    }
                }
            }

            // 不良列の表示
            cntng = ngcol.Count;
            if (cntng > 0)
            {
                string result2 = "不良列があります。" + Environment.NewLine + " " + ngcol[0];
                if (cntng > 1)
                {
                    for (int k2 = 1; k2 < cntng; k2++)
                    {
                        result2 += Environment.NewLine + " " + ngcol[k2];
                    }
                }
                CC.ShowMsg(result2, CC.MSG_EXC);
                isOK = false;
            }
            else
            {
                isOK = isOK && true;
            }

            if (isOK) CC.ShowMsg("OK", CC.MSG_INF);            //bool isOK = false;            
        }

        /// <summary> [ClearForm]メソッド
        ///  [btnClear_Click],[btnOpen_Click]から呼出し
        /// </summary>
        private void ClearForm()
        {
            // crsct構造体列要素のvdcdプロパティクリア
            for (int i1 = 0; i1 < _col; i1++)
            {  _sctcol[i1].vdecided = false;  }

            // crsct構造体行要素のvdcdプロパティクリア
            for (int j1 = 0; j1 < _row; j1++)
            {  _sctrow[j1].vdecided = false;  }

            // CellStateの初期化
            InitializeCellState();

            // _lstStateクリア
            if (_lstState != null)
            {
                _lstState.Clear();
                //_exeTry4 = false;
            }

            _decided = 0;

            // セル色の初期化
            for (int i = 0; i < _row; i++)
            {
                for (int j = 0; j < _col; j++)
                {
                    SetCellColor(i, j, SystemColors.Control);
                }
            }
            
            // [実行]ボタン有効化, [確認]ボタン無効化
            btnCheck.Enabled = false;
            btnExecute.Enabled = true;

            stlbl1.Text = String.Empty;
        }

        /// <summary> [InitializeCellStat]メソッド: CellStateの初期化
        ///  [ClearForm],[btnExecute_Click]から呼出し
        /// </summary>
        private void InitializeCellState()
        {
            for (int i = 0; i < _row; i++)
            {
                for (int j = 0; j < _col; j++)
                {
                    _cells[i, j] = CellState.Unfixed;
                }
            }
        }

        /// <summary> [PutGrid2Pbx]メソッド：Bitmapのグリッド部作成
        ///  [SetForm]から呼出し
        /// </summary>
        private void PutGrid2Pbx()
        {
            for (int i1 = 0; i1 <= _row; i1++)
            {
                for (int j1 = 0; j1 <= _col * (C_SISE + 1); j1++)
                {
                    _bpic.SetPixel(j1, i1 * (C_SISE + 1), SystemColors.ButtonShadow);
                }
            }
            for (int j2 = 0; j2 <= _col; j2++)
            {
                for (int i2 = 0; i2 <= _row * (C_SISE + 1); i2++)
                {
                    _bpic.SetPixel(j2 * (C_SISE + 1), i2, SystemColors.ButtonShadow);
                }
            }
        }

        /// <summary> [SetCellColor]メソッド
        /// </summary>
        /// <param name="vi">横方向Index</param>
        /// <param name="vj">縦方向Index</param>
        /// <param name="vcolor">設定色</param>
        private void SetCellColor(int vi, int vj, Color vcolor)
        {
            int x, y;
            for (int i = 0; i < C_SISE; i++)
            {
                for (int j = 0; j < C_SISE; j++)
                {
                    x = (C_SISE + 1) * vi + i + 1;
                    y = (C_SISE + 1) * vj + j + 1;
                    _bpic.SetPixel(y, x, vcolor);
                }
            }
            pbx1.Image = _bpic;
        }

        /// <summary> [SetForm]メソッド
        ///  [Form_Load]から呼出
        /// </summary>
        private void SetForm()
        {
            // Formの設定
            if (_col > 15)
            {
                this.Width += (_col - 15) * B;
                if (_col == 20)
                {
                    btnExecute.Left += 14;
                    btnCheck.Left += 14;
                    btnClear.Left += 14;
                    btnSave.Left += 14;
                    btnOpen.Left += 14;
                }
                else
                {
                    btnExecute.Left += 14;
                    btnCheck.Left += 14 + 7;
                    btnClear.Left += 14 + 14;
                    btnSave.Left += 14 + 21;
                    btnOpen.Left += 14 + 28;
                }
            }

            if (_row > 15)
            {
                this.Height += (_row - 15) * B;
            }

            this.Left = Convert.ToInt32((Screen.PrimaryScreen.Bounds.Width - this.Width) / 2);
            this.Top = Convert.ToInt32((Screen.PrimaryScreen.Bounds.Height - this.Height) / 2);

            // pbx1の初期化
            _bpic = new Bitmap(pbx1.Width, pbx1.Height);
            PutGrid2Pbx();
            pbx1.Image = _bpic;

            // tlpCol(TableLayoutPanel)の設定
            tlpCol.SuspendLayout();
            tlpCol.Dock = DockStyle.Fill;
            int coc = _col / 5;
            tlpCol.ColumnCount = coc;
            for (int i3 = 0; i3 < coc; i3++)
            {
                if (i3 > 0)
                    tlpCol.ColumnStyles.Add(new ColumnStyle(SizeType.Absolute, 49F));  // 変更要
                Label newLabel = new Label();
                newLabel.Font = new Font("ＭＳ ゴシック", 9F);
                newLabel.AutoSize = false;
                newLabel.Width = 49;
                newLabel.Height = 15;
                newLabel.TextAlign = ContentAlignment.BottomRight;
                newLabel.Text = ((i3 + 1) * 5).ToString();
                //newLabel.Text = "0" + (i3 + 1).ToString();
                tlpCol.Controls.Add(newLabel, i3, 0);
            }
            tlpCol.ResumeLayout();

            // tlpRow(TableLayoutPanel)の設定
            tlpRow.SuspendLayout();
            tlpRow.Dock = DockStyle.Fill;
            int roc = _row / 5;
            tlpRow.RowCount = roc;
            for (int i4 = 0; i4 < roc; i4++)
            {
                if (i4 > 0)
                    tlpRow.RowStyles.Add(new RowStyle(SizeType.Absolute, 49F));        // 変更要
                Label newLabel = new Label();
                newLabel.Height = 49;
                newLabel.Width = 29;
                newLabel.Font = new Font("ＭＳ ゴシック", 9F);
                newLabel.TextAlign = ContentAlignment.BottomRight;
                newLabel.Text = ((i4 + 1) * 5).ToString();
                //newLabel.Text = "0" + ((i4 + 1) * 5).ToString();
                tlpRow.Controls.Add(newLabel, 0, i4);
            }
            tlpRow.ResumeLayout();

            Application.DoEvents();
        }

        /// <summary> [SetSctCR]メソッド: SctCR構造体の初期化
        ///  [Form_Load]から呼出し
        /// </summary>
        private void SetSctCR()
        {
            // SctCR構造体の列要素の設定
            _sctcol = new SctCR[_col];
            for (int i1 = 0; i1 < _col; i1++)
            {
                string[] coldata = _lstCol[i1].Split(CC.SEPARATOR);
                _sctcol[i1].vdecided = false;
                int sumblack1 = 0;
                if (coldata[0].Length > 0)
                {
                    _sctcol[i1].cblock = coldata.Length;            // その列のblockの数
                    _sctcol[i1].cvalue = new int[coldata.Length];
                    for (int i2 = 0; i2 < coldata.Length; i2++)
                    {
                        _sctcol[i1].cvalue[i2] = Convert.ToInt32(coldata[i2]);   // i2番目のblockの値
                        sumblack1 += Convert.ToInt32(coldata[i2]);
                    }
                }
                _sctcol[i1].ceffective = sumblack1 + _sctcol[i1].cblock - 1;
            }

            // crsct構造体の行要素の設定
            _sctrow = new SctCR[_row];
            for (int j1 = 0; j1 < _row; j1++)
            {
                string[] rowdata = _lstRow[j1].Split(CC.SEPARATOR);
                _sctrow[j1].vdecided = false;
                int sumblack2 = 0;
                if (rowdata[0].Length > 0)
                {
                    _sctrow[j1].cblock = rowdata.Length;           // その列のblockの数
                    _sctrow[j1].cvalue = new int[rowdata.Length];
                    for (int j2 = 0; j2 < rowdata.Length; j2++)
                    {
                        _sctrow[j1].cvalue[j2] = Convert.ToInt32(rowdata[j2]);   // j2番目のblockの値
                        sumblack2 += Convert.ToInt32(rowdata[j2]);
                    }
                }
                _sctrow[j1].ceffective += sumblack2 + _sctrow[j1].cblock - 1;
            }
        }
        
        /// <summary> [Try1]メソッド：全部塗れる行・列を探す
        ///  [btnExecute_Click]から呼出し
        /// </summary>
        private void Try1()
        {
            bool changed;
            for (int i1 = 0; i1 < _row; i1++)
            {
                if (_sctrow[i1].ceffective == _col)
                {
                    int colx = 0;
                    for (int i2 = 0; i2 < _sctrow[i1].cblock; i2++)
                    {
                        if (i2 > 0)
                        {
                            changed = IsColorChanged(i1, colx, CellState.Yellow);
                            colx += 1;
                        }
                        for (int i3 = 0; i3 < _sctrow[i1].cvalue[i2]; i3++)
                        {
                            changed =
                                IsColorChanged(i1, colx, CellState.Black);
                            colx += 1;
                        }
                    }
                    _sctrow[i1].vdecided = true;
                }
            }

            for (int j1 = 0; j1 < _col; j1++)
            {
                if (_sctcol[j1].ceffective == _row)
                {
                    int rowx = 0;
                    for (int j2 = 0; j2 < _sctcol[j1].cblock; j2++)
                    {
                        if (j2 > 0)
                        {
                            changed = IsColorChanged(rowx, j1, CellState.Yellow);
                            rowx += 1;
                        }
                        for (int j3 = 0; j3 < _sctcol[j1].cvalue[j2]; j3++)
                        {
                            changed = IsColorChanged(rowx, j1, CellState.Black);
                            rowx += 1;
                        }
                    }
                    _sctcol[j1].vdecided = true;
                }
            }
        }

        /// <summary> [Try2]メソッド
        ///  [btnExecute_Click]から呼出
        /// </summary>
        private void Try2()
        {
            for (int i1 = 0; i1 < _row; i1++)
            {
                if (!_sctrow[i1].vdecided) Try2Detail(i1, CoRo.Row);
            }

            for (int j1 = 0; j1 < _col; j1++)
            {
                if (!_sctcol[j1].vdecided) Try2Detail(j1, CoRo.Column);
            }
        }

        /// <summary> [Try2Detail]メソッド
        ///  [Try2]から呼出し
        /// </summary>
        /// <param name="vn">行・列の値</param>
        /// <param name="vcr">enum CoRo</param>
        private void Try2Detail(int vn, CoRo vcr)
        {            
            var scr = new SctCR();
            int countcr;
            if (vcr == CoRo.Column)
            {
                scr = _sctcol[vn];
                countcr = _row;
            }
            else
            {
                scr = _sctrow[vn];
                countcr = _col;
            }

            if (scr.ceffective < 0) return;   // 通常は起こらない

            // 配列elm1, elm2の定義と値の設定
            int[] elm1 = new int[countcr];
            int[] elm2 = new int[countcr];
            int margin = countcr - scr.ceffective;   // 動かし代

            for (int i1 = 0; i1 < margin; i1++)
            {
                elm1[i1] = -1;    // 値なし
                //crx1 += 1;
            }
            int crx1 = margin;
            int crx2 = 0;
            for (int i2 = 0; i2 < scr.cblock; i2++)
            {
                for (int i3 = 0; i3 < scr.cvalue[i2]; i3++)
                {
                    elm1[crx1] = i2;
                    elm2[crx2] = i2;
                    crx1 += 1;
                    crx2 += 1;
                }
                if (crx1 < scr.ceffective)
                {
                    elm1[crx1] = -1;
                    crx1 += 1;
                }
                elm2[crx2] = -1;
                crx2 += 1;
            }
            for (int i4 = scr.ceffective; i4 < countcr; i4++)
            { elm2[i4] = -1; }

            bool modified;
            // 同一行・列のelm1, elm2が等しい場合、その行・列のセルは黒塗色となる
            for (int i5 = 0; i5 < countcr; i5++)
            {
                if (elm1[i5] > -1 && elm1[i5] == elm2[i5])
                {
                    if (vcr == CoRo.Column)
                        modified = IsColorChanged(i5, vn, CellState.Black);
                    else
                        modified = IsColorChanged(vn, i5, CellState.Black);
                }
            }
        }

        /// <summary> [Try4GetCombi]メソッド
        ///  [IsChangedByTry4Detail]から呼出し
        /// </summary>
        /// <param name="vscr">SctCR構造体オブジェクト</param>
        /// <param name="vcountcr">行・列数</param>
        /// <param name="vblock">blockの番号(最初に渡されるときは、0)</param>
        /// <param name="vmargin">該当行・列の動かし代</param>
        /// <param name="vstartpt">(最初に渡されるときは、0)</param>
        /// <param name="vstate">対象行・列のセル状態</param>
        /// <param name="vbasestate">このメソッドが呼び出されたときの対象行・列のセル状態</param>
        /// <param name="vresult">false:時間オーバーのとき</param>
        private void Try4GetCombi(SctCR vscr,
                                    int vcountcr,
                                    int vblock,
                                    int vmargin,
                                    int vstartpt,
                                    CellState[] vstate,
                                    CellState[] vbasestate,
                                    ref bool vresult)
        {
            if (!vresult) return;
            if (_cancel) return;

            for (int i1 = 0; i1 <= vmargin; i1++)
            {
                int block = vblock;
                CellState[] state1 = new CellState[vcountcr];
                for (int i2 = 0; i2 < vcountcr; i2++)
                {
                    state1[i2] = vstate[i2];
                }
                int startpt = vstartpt + i1;
                int i3;
                // vblock番目blockの黒セル数分、黒をつなぐ
                for (i3 = 0; i3 < vscr.cvalue[vblock]; i3++)
                {
                    // vscr.cvalue：vblock番目blockの黒セル数
                    state1[startpt + i3] = CellState.Black;
                }

                // [キャンセル]ボタンがクリックされたとき
                if (_cclfrm.IsAborting)
                {
                    _cancel = true;
                    return;
                }

                if (startpt == 0 || i1 > 0)
                    _lstState.Add(state1);
                else
                    _lstState[_lstState.Count - 1] = state1;

                // 処理時間が _ts を越えたら、その行・列の処理はやめる
                TimeSpan ts = DateTime.Now - _start;
                if (_ts < (decimal)ts.TotalSeconds)
                {
                    vresult = false;
                    _lstState.Clear();
                    return;
                }

                // 設定値との比較
                for (int i4 = 0; i4 < startpt + i3 + 1; i4++)
                {
                    if (i4 >= vcountcr)
                        break;
                    if (vbasestate[i4] != CellState.Unfixed)
                    {
                        if (state1[i4] != vbasestate[i4])
                        {
                            _lstState.RemoveAt(_lstState.Count - 1);
                            block = vscr.cblock - 1;
                            break;
                        }
                    }
                }

                if (block < vscr.cblock - 1)
                {
                    Try4GetCombi(vscr, vcountcr, vblock + 1, vmargin - i1,
                        startpt + i3 + 1, state1, vbasestate, ref vresult);
                }
            }
        }

        /// <summary> [UpdateStatusbar]：Statusbarを更新するメソッド
        ///  [IsChangedByTry4],[IsColorChanged]から呼出し
        /// </summary>
        private void UpdateStatusbar()
        {
            stlbl1.Text = "timespan：" + _ts.ToString("#0.0")
                + "秒  決定済み：" + _decided.ToString() + "/" + _total.ToString()
                + "  (" + _count.ToString() + "回目 " + _columnrow + _crnum.ToString() + ")";
            Application.DoEvents();
        }

    }
}
